

let _ = GMain.Main.init ()

let window = 
  GWindow.window ~width:1000 ~height:800 
    ~title:"ICFP 2007 - The Caml Riders - Canevas d'interface graphique " ()

let vbox = GPack.vbox ~packing:window#add ()

(* actions *)


let save () = Format.printf "Saved@."

(*

  menus

*)


let menubar = GMenu.menu_bar ~packing:vbox#pack () 
 
let menu_factory = new GMenu.factory menubar

let _ = window#add_accel_group menu_factory#accel_group

let file_menu = menu_factory#add_submenu "_File" 

let file_menu_factory = new GMenu.factory ~accel_group:menu_factory#accel_group file_menu

let save_item = file_menu_factory#add_item "_Save" 

let quit_item = file_menu_factory#add_item ~key:GdkKeysyms._Q ~callback:(fun _ -> exit 0)  "_Quit" 


(* toolbar *)

let stock_to_widget ?(size=`DIALOG) s = 
  let img = GMisc.image () 
  in img#set_stock s;
  img#coerce

let toolbar = GButton.toolbar 
		~orientation:`HORIZONTAL 
		~style:`ICONS
		~tooltips:true 
		~packing:(vbox#pack ~expand:false ~fill:false)
		  ()


let save_button = 
  toolbar#insert_button
    ~tooltip:"Save current" 
    ~callback:save
    ~icon:(stock_to_widget ~size:`LARGE_TOOLBAR `SAVE)
    ()


(* canvas *)

let fond = GdkPixbuf.from_file "helloworld.jpeg"

let image_bateau = GdkPixbuf.from_file "ship5.png"

open GnomeCanvas

let move_ship b =
  let selected = ref false 
  and cur_x = ref 0.0
  and cur_y = ref (-300.0)
  and dx = ref 0.0
  and dy = ref 0.0
  in
    fun ev ->
    begin
    match ev with
      | `BUTTON_PRESS ev ->
	  let x = GdkEvent.Button.x ev
	  and y = GdkEvent.Button.y ev
	  in
	  dx := x -. !cur_x;
	  dy := y -. !cur_y;
	  selected := true
      | `BUTTON_RELEASE _ ->
	  selected := false
      | `MOTION_NOTIFY ev ->
	  if !selected then
	    begin
	      let x = GdkEvent.Motion.x ev
	      and y = GdkEvent.Motion.y ev
	      in
	      cur_x := x -. !dx;
	      cur_y := y -. !dy;
	      b#set [`X !cur_x; `Y !cur_y]
	    end
      | _ -> ()
  end;
  true
  
(* the map *)

let init (pixmap:GDraw.pixmap) =
  pixmap#set_foreground `WHITE;
  let width, height = pixmap#size in
  pixmap#rectangle ~x:0 ~y:0 ~width ~height ~filled:true ()

let refresh (drawing:GDraw.drawable) pixmap = 
  drawing#put_pixmap ~x:0 ~y:0 pixmap#pixmap


let make_area scroll width height =
  let area = GnoCanvas.canvas  
    ~width ~height ~packing:scroll#add_with_viewport () in
  area#misc#realize ();
  area

let area = 
  let scroll = GBin.scrolled_window 
    ~hpolicy:`AUTOMATIC ~vpolicy:`AUTOMATIC ~packing:vbox#add () 
  in
  let width, height = 900,700 in
  let area = make_area scroll width height in
  let _ = GnoCanvas.pixbuf ~x:(-400.0) ~y:(-250.0) ~pixbuf:fond area#root in
  let bateau = GnoCanvas.pixbuf ~x:0.0 ~y:(-250.0) ~pixbuf:image_bateau area#root in
  let _ = bateau#connect#event ~callback:(move_ship bateau) 
  in
  area

let start ?(init=(fun () -> ())) ~enter () =
  init ();
  ignore (window#event#connect#delete ~callback:(fun _ -> exit 0));
  window#show ();
  GMain.Main.main ()


let () = 
  start ~enter:(fun () -> () (* ignore (game ())*) ) ()

(*
Local Variables: 
compile-command: "make -C .. gui"
End: 
*)
