    
open Format

type base = I | C | F | P
type dna = base list
let base_of_char = function
  | 'I' -> I
  | 'C' -> C
  | 'F' -> F
  | 'P' -> P
  | c -> Format.printf "Invalid char:'%x'@." (Char.code c);assert false

let char_of_base = function
  | I -> 'I'
  | C -> 'C'
  | F -> 'F'
  | P -> 'P'

let blos s =
  let rec loop acc i = 
    if i < 0 then acc else loop (base_of_char s.[i] :: acc) (i-1) 
  in
  loop [] (String.length s - 1)

type pat =
    | Skip of int
    | Skip24 of int
    | Par of pat list
    | Ques of dna
    | PBase of dna 

let rec quote d =
  match d with
    | [] -> []
    | h::t ->
	match h with
	  | I -> C :: quote t
	  | C -> F :: quote t
	  | F -> P :: quote t
	  | P -> I :: C :: quote t

(* nat longueur variable*)
let rec nat n =
  if n=0 then [ P ] (* zero *)
  else
    (if n mod 2 = 0 then I else C) :: nat (n/2)
(* nat longueur fixe l*)
let rec natl n l =
  if l=1 then [ P ] (* fin *)
  else
    (if n mod 2 = 0 then I else C) :: natl (n/2) (l-1)

(**)
let n = List.length(natl 42 24);;
(**)
 
let rec assemble_pat = function
  | Skip n -> I :: P :: nat n
  | Skip24 n -> I :: P :: (natl n 24)
  | Par l -> I :: I :: P :: (assemble_patlist l) @ [I ; I ; C ]
  | Ques dna -> I :: F :: F :: quote dna 
  | PBase dna -> quote dna

and assemble_patlist = function
  | [] -> []
  | p::l -> assemble_pat p @ assemble_patlist l
 
      
type tpl =
  | Index of int * int  
  | TBase of dna

let rec assemble_tpl = function
  | Index(n,l) -> 
      I :: P :: nat l @ nat n 
  | TBase dna -> quote dna

and assemble_tpllist = function
  | [] -> []
  | p::l -> assemble_tpl p @ assemble_tpllist l
 

type iteration = pat list * tpl list

let assemble_iteration (p,t) =
  (assemble_patlist p) @ [ I ; I ; C ] @
    (assemble_tpllist t) @ [ I ; I ; C]

(*
let l = nat 1234
*)

(* AST *)

let subst mark orig dest =
  [Par [Ques mark] ; PBase orig] ,
  [Index(0,0) ; TBase dest ]

  
(*
let d = assemble_iteration (subst (blos "IFPCFFP") (blos "II") (blos "IC"))
*)

(*

  quelques donnees sur endo.dna :

  start green zone : IFPICFPPCFFPP at l'offset 13615 (0x352f)

*)

let start_green_zone = blos "IFPICFPPCFFPP"
let start_bluezone_marker = blos "IFPICFPPCFIPP"
let call_zone_marker = blos "IFPICFPPCCC"

type arg = Nat of int | Bool of bool

let apply_gene gene_off gene_len args =
  (List.map
     (fun arg ->
	let a = match arg with
	  | Nat n -> natl n 24
	  | Bool b -> if b then [P] else [F]
	in
	[Par [Ques start_bluezone_marker]],
	[Index(0,0) ; TBase a])
     args)
  @
    [ [ Par [Ques call_zone_marker ; Par [ Ques call_zone_marker ]]],
      [ Index(0,0) ; TBase (nat gene_off) ; TBase (nat gene_len) 
      ; Index(1,0) ] ] 
  

let dumpstack = (blos "IIIPPPPPPP") 

let output dna file =
  let ch = open_out file in
  List.iter
    (List.iter
       (fun c -> output_char ch (char_of_base c)))
    dna;
  close_out ch

(****

let test = 
  (List.map assemble_iteration 
    (apply_gene 1234 500 [Nat 42; Bool true])) @ [ dumpstack ]

let _ = output test "../test.dna"

let genTablePageNr = 
  (List.map assemble_iteration 
    (apply_gene 0x510 0x18 [Nat 2; Bool false])) @ [ dumpstack ]

let _ = output genTablePageNr "../genTablePageNr.dna"

let addInts =
  (List.map assemble_iteration 
    (apply_gene 0x54b1ba 0x325 [Nat 2; Nat 3])) @ [ dumpstack ]
  
let _ = output addInts "../addInts.dna"

let apple =
  (List.map assemble_iteration 
    (apply_gene 0x65f785 0x3fb [])) @ [ dumpstack ]
  
let _ = output apple "../apple.dna"

let after_apple =
  (List.map assemble_iteration 
    (apply_gene (0x65f785+0x3fb) 0x100000 [])) @ [ dumpstack ]
  
let _ = output after_apple "../after_apple.dna"

let appletree =
  (List.map assemble_iteration 
    (apply_gene 0x3c870e 0x372b [])) @ [ dumpstack ]
  
let _ = output appletree "../appletree.dna"

let m_class_planet =
  (List.map assemble_iteration 
    (apply_gene 0x2ccd88 0x3c7f0 [])) @ [ dumpstack ]
  
let _ = output m_class_planet "../m_class_planet.dna"
*)


(*

  patch de gene

*)

let patch offset src dest =
  [ [ Par [Ques start_green_zone ; (*  ?<start green zone> *)
	   Skip (offset-13)        (*  13 = size of start_green_zone *)
	  ] ;
      PBase src ] ,
    [ Index(0,0) ;
      TBase dest ] ]

(* lambda x x :

  offset = 5099133 - 0xf5

  remplacer fill / garbage par addBitmap/Clip

*)

let patch_lambdaxx_disappear = (* breaks caller...*)
  List.map assemble_iteration 
    (patch (0x4d730e + 0x2640 - 0xf5) 
       (blos "IIIPIIPIIPIIICFPICFP") 
       (blos "IIIPCCPFFPIIIPFFICCF"))

let _ = output patch_lambdaxx_disappear "../patch_lambdaxx_disappear.dna"

(* SKIP *)


let calc_length_to_skip len =
  let skip_asm = assemble_pat (Skip len) @ [ I ; I ; C ] in
  let l = List.length skip_asm in
  let skip' = assemble_pat (Skip (len-l)) @ [ I ; I ; C ] in
  let l' = List.length skip' in
  if l = l' then skip',l else
    failwith "calc_length_to_skip"
  
  

let skip offset len =
  let skip_asm,skip_len = calc_length_to_skip len in
  [ [ Par [Ques start_green_zone ; (*  ?<start green zone> *)
	   Skip (offset-13)        (*  13 = size of start_green_zone *)
	  ] ;
      Skip skip_len ] ,
    [ Index(0,0) ;
      TBase skip_asm
    ] ]

let skip24prog_len = List.length (assemble_pat (Skip24 0) @ [ I ; I ; C ])

let skip_compose offset len =
  let skip_asm = 
    assemble_pat (Skip24 (len+10-skip24prog_len)) @ [ I ; I ; C ] 
  in
  [ [ Par [Ques start_green_zone ; (*  ?<start green zone> *)
	   Skip (offset-13)        (*  13 = size of start_green_zone *)
	  ] ;
      Skip (len+10) ] ,
    [ Index(0,0) ;
      TBase skip_asm ;
      TBase (blos "IIIPFFPCCP")
    ] ]

(* CAMION *)

(**)

(*
let patch_camion_color =
  List.map assemble_iteration 
     (patch (0x21edd5 + 1014) (blos "PIPIICC") (blos "PIPIICP")) 
	   
let _ = output patch_camion_color "../patch_camion_color.dna"
  *)

let patch_camion_color =
  List.map assemble_iteration 
     (patch (0x21edd5 + 879) (blos "PIPIIFC") (blos "PIPIICF")) 
	   
let _ = output patch_camion_color "../patch_camion_color.dna"

  (* lambda *)

let patch_lambda_color =
  List.map assemble_iteration 
     (patch (0x6f31b6 + 5406) (blos "PIIPIIP") (blos "PPPPPPP")) 
	   
let _ = output patch_lambda_color "../patch_lambda_color.dna"


(*
    
let patch_apple_color =
  List.map assemble_iteration 
     (patch (0x65f785 + 23) (blos "PIPIIIP") (blos "PIPIICC")) 
	   
let _ = output patch_apple_color "../patch_apple_color.dna"

*****)

(* bateau *)

let patch_bat =
  List.map assemble_iteration 
     (skip 0x592c81 (0x1c97 - 0x109)) 

let _ = output patch_bat "../patch_bat.dna"

let patch_lambda_disappear =
  List.map assemble_iteration 
(*
     (skip 0x6f31b6 (0x1a83 - 0x568)) BON mais lambda reste
*)
(*
  marche pas: laisse l'image a droite 
     (skip 0x6f31b6 (0x1a83 - 0xf5)) 
*)   
     (skip 0x6f31b6 (0x1a83 - 0xf5+10)) 

let _ = output patch_lambda_disappear "../patch_lambda_disappearr.dna"

(* ALIEN dis *)

let patch_alien_disappear =
  List.map assemble_iteration 
(*
     (skip 0x58e155 (0x1985 - 0x4c1)) 
*)
     (skip 0x58e155 (0x1985 - 0x172)) 
(*
     (skip_compose 0x58e155 (0x1985 - 0xf5)) 
*)
	   
let _ = output patch_alien_disappear "../patch_alien_disappearr.dna"


(* copie *)

let copy_backward src dest len =
  [ [ Par [Ques start_green_zone ; (*  ?<start green zone> *)
	   Skip (dest-13)        (*  13 = size of start_green_zone *)
	  ] ;
      Skip len ;
      Par [ Skip (src-dest-len)] ;
      Par [ Skip len ]
    ] ,
    [ Index(0,0) ; Index(2,0) ; Index(1,0) ; Index(2,0)
      
    ] ]
  

(*

 proc pommes vertes: 0x58822c = 5800492
   
 proc pommes rouges: 0x3c870e = 3966734

 tailles = 0x372b = 14123
*)

let vertes = quote (natl 0x58822c 24)
(*
 CCFFCFCCCFCCCCCFCCCFFCFIC, at i=2727546 (0x299e7a)
*)

let rouges = quote (natl 0x3c870e 24)
(*
 CFFFCCCCFFFCCCCFCCFFFFCIC, at i=2666738 (0x28b0f2)
*)
let taille = quote (natl 0x372b 24)
(*
FFCFCFCCFFFCFFCCCCCCCCCIC
*)
(*
copier 14123 octets de 5800492 vers 3966734 
*)
let patch_green_apples =
  List.map assemble_iteration 
    (copy_backward 5800492 3966734 14123)

let _ = output patch_green_apples "../patch_green_apples.dna"

(* ALIEN BIS : on remplace par les pommiers *)

let alien = quote (natl 0x58e155 24)
(*
FCFCFCFCFCCCCFFFCCCFFCFIC
Found at i=2695732 (0x292234)
*)
let alienlen = quote (natl 0x1985 24)
(*
FCFCCCCFFCCFFCCCCCCCCCCIC
Found at i=2695757 (29224d)
*)

(* on veut donc remplacer les caracteres 0x292234 a 0x292234+50 par
l'addresse des pommiers

(?<green>!{0x292234-13})!50 /
 0_{0} 
 CCFFCFCCCFCCCCCFCCCFFCFIC (adr gene pommes verts)
 FFCFCFCCFFFCFFCCCCCCCCCIC (taille gene pommiers)
 
*)

let patch_call offset =
  [ [ Par [Ques start_green_zone ; (*  ?<start green zone> *)
	   Skip (offset-13615-13)        (*  13 = size of start_green_zone *)
	  ] ;
      Skip 50
    ] ,
    [ Index(0,0) ; 
      TBase (blos "CCFFCFCCCFCCCCCFCCCFFCFICFFCFCFCCFFFCFFCCCCCCCCCIC")
    ] ]
  
let patch_alien =
  List.map assemble_iteration 
    (patch_call 0x292234)

let _ = output patch_alien "../patch_alien.dna"



(* enlever soucoupe *)

let patch_soucoupe_disappear =
  List.map assemble_iteration 
     (skip 0x433d7d (0x573 - 0x267 (* 0x267 ok sauf manque traits ? *))) 
(*
     (skip 0x433d7d (0x573 - 0xf5))
*)
let _ = output patch_soucoupe_disappear "../patch_saucer_disappear.dna"


