
open Types
open Misc
open Unix

type t = base list

let empty = []

let len = List.length

let baselist_of_string acc s =
  let n = String.length s in
  let rec loop acc i = 
    if i >= n then acc else loop (base_of_char s.[i] :: acc) (i+1) 
  in
  loop acc 0

let create ~prefix ~dna = 
  let lprefix = (stat prefix).st_size in
  let ldna = (stat dna).st_size in
  let n = lprefix + ldna in
  assert (n < Sys.max_string_length);
  let s = String.create n in
  let rec fill i cin = 
    match (try Some (input_char cin) with End_of_file -> None) with
      | Some c -> s.[i] <- c; fill (i+1) cin
      | None -> i
  in
  let m = fill (fill 0 (open_in prefix)) (open_in dna) in
  assert (m = n);
  List.rev (baselist_of_string [] s)

let of_string s = List.rev (baselist_of_string [] s)

let pop = function
  | [] -> raise Not_found
  | b :: dna -> b, dna

let get t ~i = List.nth t i

let rec chop i bl = match i, bl with
  | 0, bl -> bl
  | i, _ :: bl -> chop (i-1) bl
  | _ -> assert false

let rec skip t ~i = chop i t

let rec prefix acc i bl = match i, bl with
  | 0, _ | _, [] -> acc
  | i, x :: bl -> prefix (x :: acc) (i-1) bl

let sub t ~start ~stop = 
  if stop <= start then 
    [] 
  else 
    let t = chop start t in
    let acc = prefix [] (stop - start) t in
    List.rev acc

module T = struct
  type t = base list
  let length = len
  type char = base
  let get t i = get t ~i
end
module P = struct
  type t = base array
  let length = Array.length
  type char = base
  let get = Array.get
end
module KnuthMorrisPratt = Kmp.Make(T)(P)

let search_forward t ~i ~s = 
  let t = skip t ~i in
  let p = Array.of_list s in
  let index = KnuthMorrisPratt.search t p in
  index + Array.length p

let push bl t = 
  let n = List.length bl in Format.printf "(%d)@?" n;
  bl @ t



