
type t = string list

let empty = []

let len = List.fold_left (fun n s -> n + String.length s) 0

let create ~prefix ~dna = 
  let lprefix = (stat prefix).st_size in
  let ldna = (stat dna).st_size in
  let n = lprefix + ldna in
  assert (n < Sys.max_string_length);
  let s = String.create n in
  let rec fill i cin = 
    match (try Some (input_char cin) with End_of_file -> None) with
      | Some c -> s.[i] <- c; fill (i+1) cin
      | None -> i
  in
  let m = fill (fill 0 (open_in prefix)) (open_in dna) in
  assert (m = n);
  [s]

let of_string s = [s]

let rec get t ~i = 
  assert (if i<0 then (Format.printf "Got i=%d@." i; false) else true);
  match t with
  | [] -> raise Not_found
  | s :: t1 when i >= String.length s -> get t1 ~i:(i - String.length s)
  | s :: _ -> base_of_char (String.unsafe_get s i)

let rec pop = function
  | [] -> raise Not_found
  | "" :: t -> pop t
  | s :: t -> base_of_char s.[0], String.sub s 1 (String.length s - 1) :: t

let rec skip t ~i = 
  if i = 0 then t else
  match t with
  | [] -> []
  | s :: t1 when i >= String.length s -> skip t1 ~i:(i - String.length s)
  | s :: t1 -> String.sub s i (String.length s - i) :: t1

let small_length = 100000
let small_string s = String.length s < small_length

let rec compress = function
  | [] -> 
      []
  | s1 :: s2 :: sl when small_string s1 && small_string s2 ->
      compress ((s1 ^ s2) :: sl)
  | s1 :: sl ->
      s1 :: compress sl

let rec append t1 t2 = match t1, t2 with
  | [], t | t, [] -> 
      t
  | [s1], s2 :: tl2 -> 
      if small_string s1 && small_string s2 then 
	(s1 ^ s2) :: tl2 
      else 
	s1 :: t2
  | _ ->
      if List.length t1 > 1000 then compress t1 @ t2 else t1 @ t2

let append x y =
  let r = append x y in
  printf "append %d + %d = %d@." (len x) (len y) (len r);
  r

let print fmt t = List.iter (fun s -> Format.fprintf fmt "%s" s) t

let sub t ~start ~stop = 
  let rec cut i = function
    | [] -> 
	[]
    | s :: t1 when i >= String.length s -> 
	cut (i - String.length s) t1
    | s :: t1 -> 
	let n = String.length s in
	if i+stop-start <= n then 
	  [String.sub s i (stop-start)]
	else
	  String.sub s i (n-i) :: keep (stop-start-(n-i)) t1
  and keep i = function
    | [] -> 
	[]
    | s :: t1 ->
	let n = String.length s in
	if i <= n then
	  [String.sub s 0 i]
	else
	  s :: keep (i - n) t1
  in
  assert (start >= 0);
  if stop <= start then [] else cut start t

(***
let sub t ~start ~stop =
  printf "sub start=%d stop=%d =>" start stop;
  let s = sub t ~start ~stop in
  if stop-start <= 50 then printf "%a@." print s else printf "<...>@.";
  s
***)

let base b = [String.make 1 (char_of_base b)]

module T = struct
  type t = string list
  let length = len
  type char = base
  let get t i = get t ~i
end
module P = struct
  type t = base array
  let length = Array.length
  type char = base
  let get = Array.get
end
module KnuthMorrisPratt = Kmp.Make(T)(P)

let search_forward t ~i ~s = 
  let t = skip t ~i in
  let p = Array.of_list s in
  let index = KnuthMorrisPratt.search t p in
  index + Array.length p

let of_list bl =
  let s = String.create (List.length bl) in
  let rec fill i = function
    | [] -> [s]
    | b :: bl -> s.[i] <- char_of_base b; fill (i+1) bl
  in
  fill 0 bl

let to_list t =
  let rec scan acc s n i =
    if i >= n then acc else scan (base_of_char s.[i] :: acc) s n (i+1)
  in
  List.rev (List.fold_left (fun acc s -> scan acc s (String.length s) 0) [] t)

let print_start length fmt dna = 
  let t = sub ~start:0 ~stop:length dna in
  let l = to_list t in
  List.iter (fun  b -> print_base fmt b) l 

